@@ -1,4 +1,4 @@
-// Copyright 2020 Northern.tech AS
+// Copyright 2022 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -102,6 +102,43 @@ func MakeRootfsImageArtifact(version int, signed, hasScripts, augmented bool) (i
 	return MakeAnyImageArtifact(version, signed, hasScripts, &updates)
 }
 
+func MakeBootstrapArtifact() (io.Reader, error) {
+	composer := handlers.NewBootstrapArtifact()
+
+	updates := awriter.Updates{
+		Updates: []handlers.Composer{composer},
+	}
+	art := bytes.NewBuffer(nil)
+	typeInfoV3 := artifact.TypeInfoV3{}
+
+	comp := artifact.NewCompressorGzip()
+	aw := awriter.NewWriter(art, comp)
+
+	err := aw.WriteArtifact(&awriter.WriteArtifactArgs{
+		Format:  "mender",
+		Version: 3,
+		Devices: []string{"vexpress"},
+		Name:    "mender-1.1",
+		Updates: &updates,
+		Scripts: nil,
+		// Version 3 specifics:
+		Provides: &artifact.ArtifactProvides{
+			ArtifactName:  "mender-1.1",
+			ArtifactGroup: "group-1",
+		},
+		Depends: &artifact.ArtifactDepends{
+			ArtifactName:      []string{"mender-1.0"},
+			CompatibleDevices: []string{"vexpress"},
+		},
+		TypeInfoV3: &typeInfoV3,
+		Bootstrap:  true,
+	})
+	if err != nil {
+		return nil, err
+	}
+	return art, nil
+}
+
 func MakeModuleImageArtifact(signed, hasScripts bool, updateType string,
 	numFiles, numAugmentFiles int) (io.Reader, error) {
 
@@ -149,7 +186,10 @@ func MakeAnyImageArtifact(version int, signed bool,
 	if !signed {
 		aw = awriter.NewWriter(art, comp)
 	} else {
-		s := artifact.NewSigner([]byte(PrivateKey))
+		s, err := artifact.NewPKISigner([]byte(PrivateKey))
+		if err != nil {
+			return nil, err
+		}
 		aw = awriter.NewWriterSigned(art, comp, s)
 	}
 
@@ -191,6 +231,33 @@ func MakeAnyImageArtifact(version int, signed bool,
 	return art, nil
 }
 
+func mustCreateVerifier(t *testing.T, key []byte) *artifact.PKISigner {
+	v, err := artifact.NewPKIVerifier(key)
+	assert.NoError(t, err)
+	return v
+}
+
+func TestReadBootstrapArtifact(t *testing.T) {
+	art, err := MakeBootstrapArtifact()
+	assert.NoError(t, err)
+	aReader := NewReader(art)
+
+	err = aReader.ReadArtifact()
+	assert.NoError(t, err)
+
+	devComp := aReader.GetCompatibleDevices()
+	require.Len(t, devComp, 1)
+	assert.Equal(t, "vexpress", devComp[0])
+
+	assert.Len(t, aReader.GetHandlers(), 1)
+	bootstrapHandler := aReader.GetHandlers()[0]
+	assert.Equal(t, bootstrapHandler.GetUpdateType(), aReader.GetHandlers()[0].GetUpdateType())
+	assert.Equal(t, "mender-1.1", aReader.GetArtifactName())
+
+	err = aReader.ReadArtifactData()
+	assert.NoError(t, err)
+}
+
 func TestReadArtifact(t *testing.T) {
 
 	updFileContent := bytes.NewBuffer(nil)
@@ -209,15 +276,15 @@ func TestReadArtifact(t *testing.T) {
 		readError error
 	}{
 		"version 2 pass":   {2, false, rfh(), nil, nil},
-		"version 2 signed": {2, true, rfh(), artifact.NewVerifier([]byte(PublicKey)), nil},
-		"version 2 - public key error": {2, true, rfh(), artifact.NewVerifier([]byte(PublicKeyError)),
+		"version 2 signed": {2, true, rfh(), mustCreateVerifier(t, []byte(PublicKey)), nil},
+		"version 2 - public key error": {2, true, rfh(), mustCreateVerifier(t, []byte(PublicKeyError)),
 			errors.New("reader: invalid signature: crypto/rsa: verification error")},
 		// test that we do not need a verifier for signed artifact
 		"version 2 - no verifier needed for a signed artifact": {2, true, rfh(), nil, nil},
 		// Version 3 tests.
 		"version 3 - base case": {3, false, rfh(), nil, nil},
-		"version 3 - signed":    {3, true, rfh(), artifact.NewVerifier([]byte(PublicKey)), nil},
-		"version 3 - public key error": {3, true, rfh(), artifact.NewVerifier([]byte(PublicKeyError)),
+		"version 3 - signed":    {3, true, rfh(), mustCreateVerifier(t, []byte(PublicKey)), nil},
+		"version 3 - public key error": {3, true, rfh(), mustCreateVerifier(t, []byte(PublicKeyError)),
 			errors.New("readHeaderV3: reader: invalid signature: crypto/rsa: verification error")},
 	}
 
@@ -333,7 +400,7 @@ func TestReadNoHandler(t *testing.T) {
 	assert.NoError(t, err)
 
 	assert.Len(t, aReader.GetHandlers(), 1)
-	assert.Equal(t, "rootfs-image", aReader.GetHandlers()[0].GetUpdateType())
+	assert.Equal(t, "rootfs-image", *aReader.GetHandlers()[0].GetUpdateType())
 }
 
 func TestReadBroken(t *testing.T) {
@@ -476,12 +543,12 @@ func (i *installer) GetVersion() int {
 	return 3
 }
 
-func (i *installer) GetUpdateType() string {
-	return i.updateType
+func (i *installer) GetUpdateType() *string {
+	return &i.updateType
 }
 
-func (i *installer) GetUpdateOriginalType() string {
-	return ""
+func (i *installer) GetUpdateOriginalType() *string {
+	return nil
 }
 
 func (i *installer) NewInstance() handlers.Installer {
@@ -496,7 +563,7 @@ func (i *installer) ReadHeader(r io.Reader, path string, version int, augmented
 	return nil
 }
 
-func (i *installer) NewUpdateStorer(updateType string, payloadNum int) (handlers.UpdateStorer, error) {
+func (i *installer) NewUpdateStorer(updateType *string, payloadNum int) (handlers.UpdateStorer, error) {
 	return &testUpdateStorer{}, nil
 }
 
@@ -542,7 +609,7 @@ func (s *testUpdateStorer) FinishStoreUpdate() error {
 	return nil
 }
 
-func (s *testUpdateStorer) NewUpdateStorer(updateType string, payloadNum int) (handlers.UpdateStorer, error) {
+func (s *testUpdateStorer) NewUpdateStorer(updateType *string, payloadNum int) (handlers.UpdateStorer, error) {
 	return s, nil
 }
 
