@@ -1,4 +1,4 @@
-// Copyright 2021 Northern.tech AS
+// Copyright 2022 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -25,8 +25,7 @@ import (
 )
 
 const (
-	artifactOK = iota
-	errArtifactInvalidParameters
+	errArtifactInvalidParameters = iota
 	errArtifactUnsupportedVersion
 	errArtifactCreate
 	errArtifactOpen
@@ -54,6 +53,7 @@ const LatestFormatVersion = 3
 
 // Copied from urfave/cli/template.go
 // with the addition of the NOTE on the `global` `--compression flag`
+//nolint:lll
 var menderAppHelpTemplate = `NAME:
    {{.Name}}{{if .Usage}} - {{.Usage}}{{end}}
 
@@ -88,7 +88,7 @@ func applyCompressionInCommand(c *cli.Context) error {
 	// Let --compression argument work after command as well. Latest one
 	// applies.
 	if c.String("compression") != "" {
-		c.GlobalSet("compression", c.String("compression"))
+		_ = c.GlobalSet("compression", c.String("compression"))
 	}
 	return nil
 }
@@ -113,7 +113,7 @@ func getCliContext() *cli.App {
 
 	compressionFlag := cli.StringFlag{
 		Name: "compression",
-		Usage: fmt.Sprintf("Compression to use for data and header inside the artifact, "+
+		Usage: fmt.Sprintf("Compression to use for the artifact, "+
 			"currently supports: %v.", strings.Join(compressors, ", ")),
 	}
 	globalCompressionFlag := compressionFlag
@@ -127,6 +127,26 @@ func getCliContext() *cli.App {
 			"the Artifact.",
 	}
 
+	gcpKMSKeyFlag := cli.StringFlag{
+		Name: "gcp-kms-key",
+		Usage: "Resource ID of the GCP KMS key that will be used to sign " +
+			"the Artifact.",
+	}
+
+	vaultTransitKeyFlag := cli.StringFlag{
+		Name: "vault-transit-key",
+		Usage: "Key name of the Hashicorp Vault transit key that will be used to sign " +
+			"the Artifact. VAULT_TOKEN and VAULT_MOUNT_PATH environment variables " +
+			"needs to be provided. The default Hashicorp Vault URL can be overridden with " +
+			"VAULT_ADDR environment variable. If key rotation is used, the key version " +
+			"to sign can be specified with VAULT_KEY_VERSION environment variable.",
+	}
+
+	pkcs11Flag := cli.StringFlag{
+		Name:  "key-pkcs11",
+		Usage: "Use PKCS#11 interface to sign and verify artifacts",
+	}
+
 	publicKeyFlag := cli.StringFlag{
 		Name: "key, k",
 		Usage: "Full path to the public key that will be used to verify " +
@@ -172,12 +192,14 @@ func getCliContext() *cli.App {
 	// Common Payload flags
 	//
 	payloadProvides := cli.StringSliceFlag{
-		Name:  "provides, p",
-		Usage: "Generic `KEY:VALUE` which is added to the type-info -> artifact_provides section. Can be given multiple times",
+		Name: "provides, p",
+		Usage: "Generic `KEY:VALUE` which is added to the type-info -> artifact_provides section." +
+			" Can be given multiple times",
 	}
 	payloadDepends := cli.StringSliceFlag{
-		Name:  "depends, d",
-		Usage: "Generic `KEY:VALUE` which is added to the type-info -> artifact_depends section. Can be given multiple times",
+		Name: "depends, d",
+		Usage: "Generic `KEY:VALUE` which is added to the type-info -> artifact_depends section." +
+			" Can be given multiple times",
 	}
 	payloadMetaData := cli.StringFlag{
 		Name:  "meta-data, m",
@@ -227,6 +249,8 @@ func getCliContext() *cli.App {
 			Value: LatestFormatVersion,
 		},
 		privateKeyFlag,
+		gcpKMSKeyFlag,
+		vaultTransitKeyFlag,
 		cli.StringSliceFlag{
 			Name: "script, s",
 			Usage: "Full path to the state script(s). You can specify multiple " +
@@ -234,8 +258,8 @@ func getCliContext() *cli.App {
 		},
 		cli.BoolFlag{
 			Name: "legacy-rootfs-image-checksum",
-			Usage: "Use the legacy key name rootfs_image_checksum to store the providese checksum " +
-				"to the Artifact provides parameters instead of rootfs-image.checksum.",
+			Usage: "Use the legacy key name rootfs_image_checksum to store the providese checksum" +
+				" to the Artifact provides parameters instead of rootfs-image.checksum.",
 		},
 		cli.BoolFlag{
 			Name: "no-checksum-provide",
@@ -269,8 +293,9 @@ func getCliContext() *cli.App {
 		//////////////////////
 		softwareVersionNoDefault,
 		cli.StringFlag{
-			Name:  softwareNameFlag,
-			Usage: "Name of the key to store the software version: rootfs-image.NAME.version, instead of rootfs-image.version",
+			Name: softwareNameFlag,
+			Usage: "Name of the key to store the software version: rootfs-image.NAME.version," +
+				" instead of rootfs-image.version",
 		},
 		softwareVersionValue,
 		softwareFilesystem,
@@ -309,10 +334,6 @@ func getCliContext() *cli.App {
 			Usage: "Version of the artifact.",
 			Value: LatestFormatVersion,
 		},
-		cli.StringFlag{
-			Name:  "key, k",
-			Usage: "Full path to the private key that will be used to sign the artifact.",
-		},
 		cli.StringSliceFlag{
 			Name: "script, s",
 			Usage: "Full path to the state script(s). You can specify multiple " +
@@ -339,12 +360,14 @@ func getCliContext() *cli.App {
 			Usage: "Type of augmented payload. This is the same as the name of the update module",
 		},
 		cli.StringSliceFlag{
-			Name:  "augment-provides",
-			Usage: "Generic `KEY:VALUE` which is added to the augmented type-info -> artifact_provides section. Can be given multiple times",
+			Name: "augment-provides",
+			Usage: "Generic `KEY:VALUE` which is added to the augmented type-info ->" +
+				" artifact_provides section. Can be given multiple times",
 		},
 		cli.StringSliceFlag{
-			Name:  "augment-depends",
-			Usage: "Generic `KEY:VALUE` which is added to the augmented type-info -> artifact_depends section. Can be given multiple times",
+			Name: "augment-depends",
+			Usage: "Generic `KEY:VALUE` which is added to the augmented type-info ->" +
+				" artifact_depends section. Can be given multiple times",
 		},
 		cli.StringFlag{
 			Name:  "augment-meta-data",
@@ -357,19 +380,72 @@ func getCliContext() *cli.App {
 		clearsArtifactProvides,
 		noDefaultClearsArtifactProvides,
 		compressionFlag,
+		privateKeyFlag,
+		gcpKMSKeyFlag,
+		vaultTransitKeyFlag,
 		//////////////////////
 		// Sotware versions //
 		//////////////////////
 		softwareVersionNoDefault,
 		cli.StringFlag{
-			Name:  softwareNameFlag,
-			Usage: "Name of the key to store the software version: rootfs-image.NAME.version, instead of rootfs-image.PAYLOAD_TYPE.version",
+			Name: softwareNameFlag,
+			Usage: "Name of the key to store the software version: rootfs-image.NAME.version," +
+				" instead of rootfs-image.PAYLOAD_TYPE.version",
 		},
 		softwareVersionValue,
 		softwareFilesystem,
 	}
 	writeModuleCommand.Before = applyCompressionInCommand
 
+	//
+	// Write Bootstrap artifact
+	//
+	writeBootstrapArtifactCommand := cli.Command{
+		Name:   "bootstrap-artifact",
+		Action: writeBootstrapArtifact,
+		Usage:  "Writes Mender bootstrap artifact containing empty payload",
+	}
+
+	writeBootstrapArtifactCommand.CustomHelpTemplate = CustomSubcommandHelpTemplate
+
+	writeBootstrapArtifactCommand.Flags = []cli.Flag{
+		cli.StringSliceFlag{
+			Name: "device-type, t",
+			Usage: "Type of device(s) supported by the Artifact. You can specify multiple " +
+				"compatible devices providing this parameter multiple times.",
+			Required: true,
+		},
+		artifactName,
+		cli.StringFlag{
+			Name:  "output-path, o",
+			Usage: "Full path to output artifact file.",
+		},
+		cli.IntFlag{
+			Name:  "version, v",
+			Usage: "Version of the artifact.",
+			Value: LatestFormatVersion,
+		},
+		cli.BoolFlag{
+			Name:  "no-progress",
+			Usage: "Suppress the progressbar output",
+		},
+		compressionFlag,
+		clearsArtifactProvides,
+		payloadProvides,
+		payloadDepends,
+		privateKeyFlag,
+		gcpKMSKeyFlag,
+		vaultTransitKeyFlag,
+		/////////////////////////
+		// Version 3 specifics.//
+		/////////////////////////
+		artifactNameDepends,
+		artifactProvidesGroup,
+		artifactDependsGroups,
+	}
+
+	writeBootstrapArtifactCommand.Before = applyCompressionInCommand
+
 	writeCommand := cli.Command{
 		Name:     "write",
 		Usage:    "Writes artifact file.",
@@ -377,6 +453,7 @@ func getCliContext() *cli.App {
 		Subcommands: []cli.Command{
 			writeRootfsCommand,
 			writeModuleCommand,
+			writeBootstrapArtifactCommand,
 		},
 	}
 
@@ -390,7 +467,12 @@ func getCliContext() *cli.App {
 		Action:      validateArtifact,
 		UsageText:   "mender-artifact validate [options] <pathspec>",
 		Description: "This command validates artifact file provided by pathspec.",
-		Flags:       []cli.Flag{publicKeyFlag},
+		Flags: []cli.Flag{
+			publicKeyFlag,
+			gcpKMSKeyFlag,
+			vaultTransitKeyFlag,
+			pkcs11Flag,
+		},
 	}
 
 	//
@@ -405,6 +487,9 @@ func getCliContext() *cli.App {
 		Description: "This command validates artifact file provided by pathspec.",
 		Flags: []cli.Flag{
 			publicKeyFlag,
+			gcpKMSKeyFlag,
+			vaultTransitKeyFlag,
+			pkcs11Flag,
 			cli.BoolFlag{
 				Name:  "no-progress",
 				Usage: "Suppress the progressbar output",
@@ -426,6 +511,8 @@ func getCliContext() *cli.App {
 	}
 	sign.Flags = []cli.Flag{
 		privateKeyFlag,
+		gcpKMSKeyFlag,
+		vaultTransitKeyFlag,
 		cli.StringFlag{
 			Name: "output-path, o",
 			Usage: "Full path to output signed artifact file; " +
@@ -435,18 +522,20 @@ func getCliContext() *cli.App {
 			Name:  "force, f",
 			Usage: "Force creating new signature if the artifact is already signed",
 		},
+		pkcs11Flag,
 	}
 
 	//
 	// modify existing
 	//
 	modify := cli.Command{
-		Name:        "modify",
-		Usage:       "Modifies image or artifact file.",
-		Category:    "Artifact modification",
-		Action:      modifyArtifact,
-		UsageText:   "mender-artifact modify [options] <pathspec>",
-		Description: "This command modifies existing image or artifact file provided by pathspec. NOTE: Currently only ext4 payloads can be modified",
+		Name:      "modify",
+		Usage:     "Modifies image or artifact file.",
+		Category:  "Artifact modification",
+		Action:    modifyArtifact,
+		UsageText: "mender-artifact modify [options] <pathspec>",
+		Description: "This command modifies existing image or artifact file provided by pathspec." +
+			" NOTE: Currently only ext4 payloads can be modified",
 	}
 
 	modify.Flags = []cli.Flag{
@@ -488,11 +577,13 @@ func getCliContext() *cli.App {
 			Usage: "Full path to the tenant token that will be injected into modified file.",
 		},
 		privateKeyFlag,
+		gcpKMSKeyFlag,
+		vaultTransitKeyFlag,
 		compressionFlag,
 	}
 	modify.Before = func(c *cli.Context) error {
 		if c.String("name") != "" {
-			c.Set("artifact-name", c.String("name"))
+			_ = c.Set("artifact-name", c.String("name"))
 		}
 		return applyCompressionInCommand(c)
 	}
@@ -511,6 +602,8 @@ func getCliContext() *cli.App {
 	copy.Flags = []cli.Flag{
 		compressionFlag,
 		privateKeyFlag,
+		gcpKMSKeyFlag,
+		vaultTransitKeyFlag,
 	}
 
 	cat := cli.Command{
@@ -522,11 +615,13 @@ func getCliContext() *cli.App {
 	}
 
 	install := cli.Command{
-		Name:        "install",
-		Usage:       "install -m <permissions> <hostfile> [artifact|sdimg|uefiimg]:<filepath> or install -d [artifact|sdimg|uefiimg]:<directory>",
-		Description: "Installs a directory, or a file from the host filesystem, to the artifact or sdimg.",
-		Category:    "Artifact modification",
-		Action:      Install,
+		Name: "install",
+		Usage: "install -m <permissions> <hostfile> [artifact|sdimg|uefiimg]:<filepath> or" +
+			" install -d [artifact|sdimg|uefiimg]:<directory>",
+		Description: "Installs a directory, or a file from the host filesystem, to the artifact" +
+			" or sdimg.",
+		Category: "Artifact modification",
+		Action:   Install,
 	}
 
 	install.Flags = []cli.Flag{
@@ -559,12 +654,13 @@ func getCliContext() *cli.App {
 	// dump
 	//
 	dumpCommand := cli.Command{
-		Name:        "dump",
-		Usage:       "Dump contents from Artifacts",
-		ArgsUsage:   "<Artifact>",
-		Description: "Dump various raw files from the Artifact. These can be used to create a new Artifact with the same components.",
-		Category:    "Artifact inspection",
-		Action:      DumpCommand,
+		Name:      "dump",
+		Usage:     "Dump contents from Artifacts",
+		ArgsUsage: "<Artifact>",
+		Description: "Dump various raw files from the Artifact. These can be used to create a new" +
+			" Artifact with the same components.",
+		Category: "Artifact inspection",
+		Action:   DumpCommand,
 	}
 	dumpCommand.Flags = []cli.Flag{
 		cli.StringFlag{
@@ -572,20 +668,26 @@ func getCliContext() *cli.App {
 			Usage: "Dump all included files in the first payload into given folder",
 		},
 		cli.StringFlag{
-			Name:  "meta-data",
-			Usage: "Dump the contents of the meta-data field in the first payload into given folder",
+			Name: "meta-data",
+			Usage: "Dump the contents of the meta-data field in the first payload into given" +
+				" folder",
 		},
 		cli.StringFlag{
 			Name:  "scripts",
 			Usage: "Dump all included state scripts into given folder",
 		},
 		cli.BoolFlag{
-			Name:  "print-cmdline",
-			Usage: "Print the command line that can recreate the same Artifact with the components being dumped. If all the components are being dumped, a nearly identical Artifact can be created. Note that timestamps will cause the checksum of the Artifact to be different, and signatures can not be recreated this way. The command line will only use long option names.",
+			Name: "print-cmdline",
+			Usage: "Print the command line that can recreate the same Artifact with the" +
+				" components being dumped. If all the components are being dumped, a nearly" +
+				" identical Artifact can be created. Note that timestamps will cause the checksum" +
+				" of the Artifact to be different, and signatures can not be recreated this way." +
+				" The command line will only use long option names.",
 		},
 		cli.BoolFlag{
-			Name:  "print0-cmdline",
-			Usage: "Same as 'print-cmdline', except that the arguments are separated by a null character (0x00).",
+			Name: "print0-cmdline",
+			Usage: "Same as 'print-cmdline', except that the arguments are separated by a null" +
+				" character (0x00).",
 		},
 	}
 
