@@ -1,4 +1,4 @@
-// Copyright 2021 Northern.tech AS
+// Copyright 2022 Northern.tech AS
 //
 //    Licensed under the Apache License, Version 2.0 (the "License");
 //    you may not use this file except in compliance with the License.
@@ -30,14 +30,15 @@ import (
 	"io"
 	"io/ioutil"
 
+	"github.com/pkg/errors"
+	"github.com/urfave/cli"
+
 	"github.com/mendersoftware/mender-artifact/artifact"
 	"github.com/mendersoftware/mender-artifact/artifact/stage"
 	"github.com/mendersoftware/mender-artifact/awriter"
 	"github.com/mendersoftware/mender-artifact/cli/util"
 	"github.com/mendersoftware/mender-artifact/handlers"
 	"github.com/mendersoftware/mender-artifact/utils"
-	"github.com/pkg/errors"
-	"github.com/urfave/cli"
 )
 
 func writeRootfsImageChecksum(rootfsFilename string,
@@ -45,7 +46,10 @@ func writeRootfsImageChecksum(rootfsFilename string,
 	chk := artifact.NewWriterChecksum(ioutil.Discard)
 	payload, err := os.Open(rootfsFilename)
 	if err != nil {
-		return cli.NewExitError(fmt.Sprintf("Failed to open the payload file: %q", rootfsFilename), 1)
+		return cli.NewExitError(
+			fmt.Sprintf("Failed to open the payload file: %q", rootfsFilename),
+			1,
+		)
 	}
 	if _, err = io.Copy(chk, payload); err != nil {
 		return cli.NewExitError("Failed to generate the checksum for the payload", 1)
@@ -75,11 +79,16 @@ func writeRootfsImageChecksum(rootfsFilename string,
 
 func validateInput(c *cli.Context) error {
 	// Version 2 and 3 validation.
+	fileMissing := false
+	if c.Command.Name != "bootstrap-artifact" {
+		if len(c.String("file")) == 0 {
+			fileMissing = true
+		}
+	}
 	if len(c.StringSlice("device-type")) == 0 ||
-		len(c.String("artifact-name")) == 0 ||
-		len(c.String("file")) == 0 {
+		len(c.String("artifact-name")) == 0 || fileMissing {
 		return cli.NewExitError(
-			"must provide `device-type`, `artifact-name` and `update`",
+			"must provide `device-type`, `artifact-name` and `file`",
 			errArtifactInvalidParameters,
 		)
 	}
@@ -99,11 +108,20 @@ func createRootfsFromSSH(c *cli.Context) (string, error) {
 		return rootfsFilename, cli.NewExitError("SSH error: "+err.Error(), 1)
 	}
 
-	// run fsck
+	// check for blkid and get filesystem type
 	fstype, err := imgFilesystemType(rootfsFilename)
 	if err != nil {
-		return rootfsFilename, cli.NewExitError("imgFilesystemType error: "+err.Error(), errArtifactCreate)
+		if err == errBlkidNotFound {
+			Log.Warnf("Skipping running fsck on the Artifact: %v", err)
+			return rootfsFilename, nil
+		}
+		return rootfsFilename, cli.NewExitError(
+			"imgFilesystemType error: "+err.Error(),
+			errArtifactCreate,
+		)
 	}
+
+	// run fsck
 	switch fstype {
 	case fat:
 		err = runFsck(rootfsFilename, "vfat")
@@ -120,10 +138,125 @@ func createRootfsFromSSH(c *cli.Context) (string, error) {
 	return rootfsFilename, nil
 }
 
+func makeEmptyUpdates(ctx *cli.Context) (*awriter.Updates, error) {
+	handler := handlers.NewBootstrapArtifact()
+
+	dataFiles := make([](*handlers.DataFile), 0)
+	if err := handler.SetUpdateFiles(dataFiles); err != nil {
+		return nil, cli.NewExitError(
+			err,
+			1,
+		)
+	}
+
+	upd := &awriter.Updates{
+		Updates: []handlers.Composer{handler},
+	}
+	return upd, nil
+}
+
+func writeBootstrapArtifact(c *cli.Context) error {
+	comp, err := artifact.NewCompressorFromId(c.GlobalString("compression"))
+	if err != nil {
+		return cli.NewExitError(
+			"compressor '"+c.GlobalString("compression")+"' is not supported: "+err.Error(),
+			1,
+		)
+	}
+
+	if err := validateInput(c); err != nil {
+		Log.Error(err.Error())
+		return err
+	}
+
+	// set the default name
+	name := "artifact.mender"
+	if len(c.String("output-path")) > 0 {
+		name = c.String("output-path")
+	}
+	version := c.Int("version")
+
+	Log.Debugf("creating bootstrap artifact [%s], version: %d", name, version)
+
+	f, err := os.Create(name + ".tmp")
+	if err != nil {
+		return cli.NewExitError(
+			"can not create bootstrap artifact file: "+err.Error(),
+			errArtifactCreate,
+		)
+	}
+	defer func() {
+		f.Close()
+		// in case of success `.tmp` suffix will be removed and below
+		// will not remove valid artifact
+		os.Remove(name + ".tmp")
+	}()
+
+	aw, err := artifactWriter(c, comp, f, version)
+	if err != nil {
+		return cli.NewExitError(err.Error(), 1)
+	}
+
+	depends := artifact.ArtifactDepends{
+		ArtifactName:      c.StringSlice("artifact-name-depends"),
+		CompatibleDevices: c.StringSlice("device-type"),
+		ArtifactGroup:     c.StringSlice("depends-groups"),
+	}
+
+	provides := artifact.ArtifactProvides{
+		ArtifactName:  c.String("artifact-name"),
+		ArtifactGroup: c.String("provides-group"),
+	}
+
+	upd, err := makeEmptyUpdates(c)
+	if err != nil {
+		return err
+	}
+
+	typeInfoV3, _, err := makeTypeInfo(c)
+	if err != nil {
+		return err
+	}
+
+	if !c.Bool("no-progress") {
+		ctx, cancel := context.WithCancel(context.Background())
+		go reportProgress(ctx, aw.State)
+		defer cancel()
+		aw.ProgressWriter = utils.NewProgressWriter()
+	}
+
+	err = aw.WriteArtifact(
+		&awriter.WriteArtifactArgs{
+			Format:     "mender",
+			Version:    version,
+			Devices:    c.StringSlice("device-type"),
+			Name:       c.String("artifact-name"),
+			Updates:    upd,
+			Scripts:    nil,
+			Depends:    &depends,
+			Provides:   &provides,
+			TypeInfoV3: typeInfoV3,
+			Bootstrap:  true,
+		})
+	if err != nil {
+		return cli.NewExitError(err.Error(), 1)
+	}
+
+	f.Close()
+	err = os.Rename(name+".tmp", name)
+	if err != nil {
+		return cli.NewExitError(err.Error(), 1)
+	}
+	return nil
+}
+
 func writeRootfs(c *cli.Context) error {
 	comp, err := artifact.NewCompressorFromId(c.GlobalString("compression"))
 	if err != nil {
-		return cli.NewExitError("compressor '"+c.GlobalString("compression")+"' is not supported: "+err.Error(), 1)
+		return cli.NewExitError(
+			"compressor '"+c.GlobalString("compression")+"' is not supported: "+err.Error(),
+			1,
+		)
 	}
 
 	if err := validateInput(c); err != nil {
@@ -179,7 +312,7 @@ func writeRootfs(c *cli.Context) error {
 		os.Remove(name + ".tmp")
 	}()
 
-	aw, err := artifactWriter(comp, f, c.String("key"), version)
+	aw, err := artifactWriter(c, comp, f, version)
 	if err != nil {
 		return cli.NewExitError(err.Error(), 1)
 	}
@@ -208,7 +341,10 @@ func writeRootfs(c *cli.Context) error {
 	if !c.Bool("no-checksum-provide") {
 		legacy := c.Bool("legacy-rootfs-image-checksum")
 		if err = writeRootfsImageChecksum(rootfsFilename, typeInfoV3, legacy); err != nil {
-			return cli.NewExitError(errors.Wrap(err, "Failed to write the `rootfs-image.checksum` to the artifact"), 1)
+			return cli.NewExitError(
+				errors.Wrap(err, "Failed to write the `rootfs-image.checksum` to the artifact"),
+				1,
+			)
 		}
 	}
 
@@ -246,7 +382,7 @@ func writeRootfs(c *cli.Context) error {
 func reportProgress(c context.Context, state chan string) {
 	fmt.Fprintln(os.Stderr, "Writing Artifact...")
 	str := fmt.Sprintf("%-20s\t", <-state)
-	fmt.Fprintf(os.Stderr, str)
+	fmt.Fprint(os.Stderr, str)
 	for {
 		select {
 		case str = <-state:
@@ -256,7 +392,7 @@ func reportProgress(c context.Context, state chan string) {
 			} else {
 				fmt.Fprintf(os.Stderr, "\033[1;32m\u2713\033[0m\n")
 				str = fmt.Sprintf("%-20s\t", str)
-				fmt.Fprintf(os.Stderr, str)
+				fmt.Fprint(os.Stderr, str)
 			}
 		case <-c.Done():
 			return
@@ -264,18 +400,18 @@ func reportProgress(c context.Context, state chan string) {
 	}
 }
 
-func artifactWriter(comp artifact.Compressor, f *os.File, key string,
+func artifactWriter(c *cli.Context, comp artifact.Compressor, f *os.File,
 	ver int) (*awriter.Writer, error) {
-	if key != "" {
+	privateKey, err := getKey(c)
+	if err != nil {
+		return nil, err
+	}
+	if privateKey != nil {
 		if ver == 0 {
 			// check if we are having correct version
 			return nil, errors.New("can not use signed artifact with version 0")
 		}
-		privateKey, err := getKey(key)
-		if err != nil {
-			return nil, err
-		}
-		return awriter.NewWriterSigned(f, comp, artifact.NewSigner(privateKey)), nil
+		return awriter.NewWriterSigned(f, comp, privateKey), nil
 	}
 	return awriter.NewWriter(f, comp), nil
 }
@@ -302,7 +438,12 @@ func makeUpdates(ctx *cli.Context) (*awriter.Updates, error) {
 	for _, file := range ctx.StringSlice("file") {
 		dataFiles = append(dataFiles, &handlers.DataFile{Name: file})
 	}
-	handler.SetUpdateFiles(dataFiles)
+	if err := handler.SetUpdateFiles(dataFiles); err != nil {
+		return nil, cli.NewExitError(
+			err,
+			1,
+		)
+	}
 
 	upd := &awriter.Updates{
 		Updates: []handlers.Composer{handler},
@@ -314,7 +455,12 @@ func makeUpdates(ctx *cli.Context) (*awriter.Updates, error) {
 		for _, file := range ctx.StringSlice("augment-file") {
 			dataFiles = append(dataFiles, &handlers.DataFile{Name: file})
 		}
-		augmentHandler.SetUpdateAugmentFiles(dataFiles)
+		if err := augmentHandler.SetUpdateAugmentFiles(dataFiles); err != nil {
+			return nil, cli.NewExitError(
+				err,
+				1,
+			)
+		}
 		upd.Augments = []handlers.Composer{augmentHandler}
 	}
 
@@ -374,8 +520,13 @@ func makeTypeInfo(ctx *cli.Context) (*artifact.TypeInfoV3, *artifact.TypeInfoV3,
 		return nil, nil, err
 	}
 
+	var typeInfo *string
+	if ctx.Command.Name != "bootstrap-artifact" {
+		typeFlag := ctx.String("type")
+		typeInfo = &typeFlag
+	}
 	typeInfoV3 := &artifact.TypeInfoV3{
-		Type:                   ctx.String("type"),
+		Type:                   typeInfo,
 		ArtifactDepends:        typeInfoDepends,
 		ArtifactProvides:       typeInfoProvides,
 		ClearsArtifactProvides: clearsArtifactProvides,
@@ -395,8 +546,9 @@ func makeTypeInfo(ctx *cli.Context) (*artifact.TypeInfoV3, *artifact.TypeInfoV3,
 		return typeInfoV3, nil, nil
 	}
 
+	augmentType := ctx.String("augment-type")
 	augmentTypeInfoV3 := &artifact.TypeInfoV3{
-		Type:             ctx.String("augment-type"),
+		Type:             &augmentType,
 		ArtifactDepends:  augmentTypeInfoDepends,
 		ArtifactProvides: augmentTypeInfoProvides,
 	}
@@ -404,13 +556,20 @@ func makeTypeInfo(ctx *cli.Context) (*artifact.TypeInfoV3, *artifact.TypeInfoV3,
 	return typeInfoV3, augmentTypeInfoV3, nil
 }
 
-func getSoftwareVersion(artifactName, softwareFilesystem, softwareName, softwareNameDefault, softwareVersion string, noDefaultSoftwareVersion bool) map[string]string {
+func getSoftwareVersion(
+	artifactName,
+	softwareFilesystem,
+	softwareName,
+	softwareNameDefault,
+	softwareVersion string,
+	noDefaultSoftwareVersion bool,
+) map[string]string {
 	result := map[string]string{}
 	softwareVersionName := "rootfs-image"
 	if softwareFilesystem != "" {
 		softwareVersionName = softwareFilesystem
 	}
-	if noDefaultSoftwareVersion == false {
+	if !noDefaultSoftwareVersion {
 		if softwareName == "" {
 			softwareName = softwareNameDefault
 		}
@@ -429,12 +588,13 @@ func getSoftwareVersion(artifactName, softwareFilesystem, softwareName, software
 
 // applySoftwareVersionToTypeInfoProvides returns a new mapping, enriched with provides
 // for the software version; the mapping provided as argument is not modified
-func applySoftwareVersionToTypeInfoProvides(ctx *cli.Context, typeInfoProvides artifact.TypeInfoProvides) artifact.TypeInfoProvides {
+func applySoftwareVersionToTypeInfoProvides(
+	ctx *cli.Context,
+	typeInfoProvides artifact.TypeInfoProvides,
+) artifact.TypeInfoProvides {
 	result := make(map[string]string)
-	if typeInfoProvides != nil {
-		for key, value := range typeInfoProvides {
-			result[key] = value
-		}
+	for key, value := range typeInfoProvides {
+		result[key] = value
 	}
 	artifactName := ctx.String("artifact-name")
 	softwareFilesystem := ctx.String(softwareFilesystemFlag)
@@ -443,9 +603,19 @@ func applySoftwareVersionToTypeInfoProvides(ctx *cli.Context, typeInfoProvides a
 	if ctx.Command.Name == "module-image" {
 		softwareNameDefault = ctx.String("type")
 	}
+	if ctx.Command.Name == "bootstrap-artifact" {
+		return result
+	}
 	softwareVersion := ctx.String(softwareVersionFlag)
 	noDefaultSoftwareVersion := ctx.Bool(noDefaultSoftwareVersionFlag)
-	if softwareVersionMapping := getSoftwareVersion(artifactName, softwareFilesystem, softwareName, softwareNameDefault, softwareVersion, noDefaultSoftwareVersion); len(softwareVersionMapping) > 0 {
+	if softwareVersionMapping := getSoftwareVersion(
+		artifactName,
+		softwareFilesystem,
+		softwareName,
+		softwareNameDefault,
+		softwareVersion,
+		noDefaultSoftwareVersion,
+	); len(softwareVersionMapping) > 0 {
 		for key, value := range softwareVersionMapping {
 			if result[key] == "" || softwareVersionOverridesProvides(ctx, key) {
 				result[key] = value
@@ -472,14 +642,17 @@ func softwareVersionOverridesProvides(ctx *cli.Context, key string) bool {
 	} else if len(softwareIndexes) == 0 {
 		return false
 	} else {
-		return softwareIndexes[len(softwareIndexes)-1][0] > providesIndexes[len(providesIndexes)-1][0]
+		return softwareIndexes[len(softwareIndexes)-1][0] >
+			providesIndexes[len(providesIndexes)-1][0]
 	}
 }
 
 func makeClearsArtifactProvides(ctx *cli.Context) ([]string, error) {
 	list := ctx.StringSlice(clearsProvidesFlag)
 
-	if ctx.Bool(noDefaultClearsProvidesFlag) || ctx.Bool(noDefaultSoftwareVersionFlag) {
+	if ctx.Bool(noDefaultClearsProvidesFlag) ||
+		ctx.Bool(noDefaultSoftwareVersionFlag) ||
+		ctx.Command.Name == "bootstrap-artifact" {
 		return list, nil
 	}
 
@@ -510,7 +683,9 @@ func makeClearsArtifactProvides(ctx *cli.Context) ([]string, error) {
 	} else if ctx.Command.Name == "module-image" {
 		softwareName = ctx.String("type") + "."
 	} else {
-		return nil, errors.New("Unknown write command in makeClearsArtifactProvides(), this is a bug.")
+		return nil, errors.New(
+			"Unknown write command in makeClearsArtifactProvides(), this is a bug.",
+		)
 	}
 
 	defaultCap := fmt.Sprintf("%s.%s*", softwareFilesystem, softwareName)
@@ -563,7 +738,10 @@ func makeMetaData(ctx *cli.Context) (map[string]interface{}, map[string]interfac
 func writeModuleImage(ctx *cli.Context) error {
 	comp, err := artifact.NewCompressorFromId(ctx.GlobalString("compression"))
 	if err != nil {
-		return cli.NewExitError("compressor '"+ctx.GlobalString("compression")+"' is not supported: "+err.Error(), 1)
+		return cli.NewExitError(
+			"compressor '"+ctx.GlobalString("compression")+"' is not supported: "+err.Error(),
+			1,
+		)
 	}
 
 	// set the default name
@@ -601,7 +779,7 @@ func writeModuleImage(ctx *cli.Context) error {
 		os.Remove(name + ".tmp")
 	}()
 
-	aw, err := artifactWriter(comp, f, ctx.String("key"), version)
+	aw, err := artifactWriter(ctx, comp, f, version)
 	if err != nil {
 		return cli.NewExitError(err.Error(), 1)
 	}
@@ -714,7 +892,8 @@ func getDeviceSnapshot(c *cli.Context) (string, error) {
 	// and hence impossible to detect).
 	// When user id is 0 do not bother with sudo.
 	args = append(args, "/bin/sh", "-c",
-		`'[ $(id -u) -eq 0 ] || sudo_cmd="sudo -S"; $sudo_cmd /bin/sh -c "echo `+sshInitMagic+`; mender snapshot dump" | cat'`)
+		`'[ $(id -u) -eq 0 ] || sudo_cmd="sudo -S"; $sudo_cmd /bin/sh -c "echo `+sshInitMagic+
+			`; mender snapshot dump" | cat'`)
 
 	cmd := exec.Command("ssh", args...)
 
@@ -756,14 +935,14 @@ func getDeviceSnapshot(c *cli.Context) (string, error) {
 	// Wait for 60 seconds for ssh to establish connection
 	err = waitForBufferSignal(stdout, os.Stdout, sshInitMagic, 2*time.Minute)
 	if err != nil {
-		cmd.Process.Kill()
+		_ = cmd.Process.Kill()
 		return "", errors.Wrap(err,
 			"Error waiting for ssh session to be established.")
 	}
 
 	_, err = recvSnapshot(f, stdout)
 	if err != nil {
-		cmd.Process.Kill()
+		_ = cmd.Process.Kill()
 		return "", err
 	}
 
@@ -849,30 +1028,17 @@ func recvSnapshot(dst io.Writer, src io.Reader) (int64, error) {
 	return written, nil
 }
 
-func sizeStr(bytes int64) string {
-	tmp := bytes
-	var i int
-	var suffixes = [...]string{"B", "KiB", "MiB", "GiB", "TiB"}
-	for i = 0; i < len(suffixes); i++ {
-		if (tmp / 1024) == 0 {
-			break
-		}
-		tmp /= 1024
-	}
-	return fmt.Sprintf("%d %s", tmp, suffixes[i])
-}
-
 func removeOnPanic(filename string) {
 	if r := recover(); r != nil {
 		err := os.Remove(filename)
 		if err != nil {
-			switch r.(type) {
+			switch v := r.(type) {
 			case string:
-				err = errors.Wrap(errors.
-					New(r.(string)), err.Error())
+				err = errors.Wrap(errors.New(v), err.Error())
 				panic(err)
 			case error:
-				err = errors.Wrap(r.(error), err.Error())
+				err = errors.Wrap(v, err.Error())
+				panic(err)
 			default:
 				panic(r)
 			}
